# $Id: /local/datetime/modules/DateTime-Util-Astro/trunk/lib/DateTime/Util/Astro.pm 11785 2007-05-29T22:28:00.218926Z daisuke  $
#
# Copyright (c) 2004-2007 Daisuke Maki <daisuke@endeworks.jp>

package DateTime::Util::Astro;
use strict;
our $VERSION = '0.11001';

1;

__END__

=head1 NAME

DateTime::Util::Astro - Utilities for Astronomical Calculations

=head1 DESCRIPTION

See DateTime::Util::Astro::Moon and DateTime::Util::Astro::Sun

=head1 AUTHOR

Copyright (c) 2004-2007 Daisuke Maki E<lt>daisuke@endeworks.jpE<gt>

=cut
